//
// Copyright (c) Tiny Technologies, Inc. All rights reserved.
// Licensed under the LGPL or a commercial license.
// For LGPL see License.txt in the project root for license information.
// For commercial licenses see https://www.tiny.cloud/
//

//
// Media Figure object
//

.mce-content-body {
  // Figure alignment
  figure.align-left {
    float: left;
  }

  figure.align-right {
    float: right;
  }

  figure.image.align-center {
    display: table;
    margin-left: auto;
    margin-right: auto;
  }
}
