//
// Copyright (c) Tiny Technologies, Inc. All rights reserved.
// Licensed under the LGPL or a commercial license.
// For LGPL see License.txt in the project root for license information.
// For commercial licenses see https://www.tiny.cloud/
//

//
// Autocomplete
//

.tox {
  // cap the width of the autocompleter for horizontal cases e.g. emoji picker
  .tox-autocompleter {
    max-width: 25em;
  }

  .tox-autocompleter .tox-menu {
    max-width: 25em;
  }

  .tox-autocompleter .tox-autocompleter-highlight {
    font-weight: @font-weight-bold;
  }
}
