//
// Copyright (c) Tiny Technologies, Inc. All rights reserved.
// Licensed under the LGPL or a commercial license.
// For LGPL see License.txt in the project root for license information.
// For commercial licenses see https://www.tiny.cloud/
//

//
// Source code
//

// TODO - Verify if this class is needed
.tox {
  // Wrapper for Code Editor to enable scroll within `.tox-well`
  .tox-source-code {
    overflow: auto;
  }
}
