//
// Copyright (c) Tiny Technologies, Inc. All rights reserved.
// Licensed under the LGPL or a commercial license.
// For LGPL see License.txt in the project root for license information.
// For commercial licenses see https://www.tiny.cloud/
//

//
// Anchorbar
// An element that lives underneath the toolbar so that we have
// something on the page, which is 0px thick, so invisible, that
// we can measure so we can put elements like the notification
// banner and inline dialogs below it.
//

.tox {
  .tox-anchorbar {
    display: flex;
    flex: 0 0 auto;
  }
}
