//
// Copyright (c) Tiny Technologies, Inc. All rights reserved.
// Licensed under the LGPL or a commercial license.
// For LGPL see License.txt in the project root for license information.
// For commercial licenses see https://www.tiny.cloud/
//

//
// (Aspect ratio) Lock
//

.tox {
  .tox-lock.tox-locked .tox-lock-icon__unlock,
  .tox-lock:not(.tox-locked) .tox-lock-icon__lock {
    display: none;
  }
}
