//
// Copyright (c) Tiny Technologies, Inc. All rights reserved.
// Licensed under the LGPL or a commercial license.
// For LGPL see License.txt in the project root for license information.
// For commercial licenses see https://www.tiny.cloud/
//

//
// Dialog root document blocker
//

// Added to the body to disable scrolling when dialogs are open
body.tox-dialog__disable-scroll {
  overflow: hidden;
}
