//
// Copyright (c) Tiny Technologies, Inc. All rights reserved.
// Licensed under the LGPL or a commercial license.
// For LGPL see License.txt in the project root for license information.
// For commercial licenses see https://www.tiny.cloud/
//

//
// Shadow DOM styles for the root page.
//
// This file contains styles specific the root document, such as blocking scrolling or setting the
// z-index on the shadow-host. It should not contain any styling for UI components or editor content.
//

@import 'globals/global-variables';

@import 'components/dialog/blocker';
@import 'components/fullscreen/fullscreen';
