//
// Copyright (c) Tiny Technologies, Inc. All rights reserved.
// Licensed under the LGPL or a commercial license.
// For LGPL see License.txt in the project root for license information.
// For commercial licenses see https://www.tiny.cloud/
//

.tinymce-mobile-outer-container .tinymce-mobile-disabled-mask {
  .mixin-flex-and-centre;

  background: fadeout(@editor-mask-background-color, 50%);
  height: 100%;
  position: absolute;
  top: 0;
  width: 100%;

  .tinymce-mobile-content-container {
    align-items: center;
    border-radius: 50%;
    display: flex;
    flex-direction: column;
    font-family: sans-serif;
    font-size: 1em;
    justify-content: space-between;

    @import "mask-section.less";
  }
}
