//
// Copyright (c) Tiny Technologies, Inc. All rights reserved.
// Licensed under the LGPL or a commercial license.
// For LGPL see License.txt in the project root for license information.
// For commercial licenses see https://www.tiny.cloud/
//

.tinymce-mobile-icon-arrow-back::before {
  content: "\e5cd";
}

.tinymce-mobile-icon-image::before {
  content: "\e412";
}

.tinymce-mobile-icon-cancel-circle::before {
  content: "\e5c9";
}

.tinymce-mobile-icon-full-dot::before {
  content: "\e061";
}

.tinymce-mobile-icon-align-center::before {
  content: "\e234";
}

.tinymce-mobile-icon-align-left::before {
  content: "\e236";
}

.tinymce-mobile-icon-align-right::before {
  content: "\e237";
}

.tinymce-mobile-icon-bold::before {
  content: "\e238";
}

.tinymce-mobile-icon-italic::before {
  content: "\e23f";
}

.tinymce-mobile-icon-unordered-list::before {
  content: "\e241";
}

.tinymce-mobile-icon-ordered-list::before {
  content: "\e242";
}

.tinymce-mobile-icon-font-size::before {
  content: "\e245";
}

.tinymce-mobile-icon-underline::before {
  content: "\e249";
}

.tinymce-mobile-icon-link::before {
  content: "\e157";
}

.tinymce-mobile-icon-unlink::before {
  content: "\eca2";
}

.tinymce-mobile-icon-color::before {
  content: "\e891";
}

.tinymce-mobile-icon-previous::before {
  content: "\e314";
}

.tinymce-mobile-icon-next::before {
  content: "\e315";
}

.tinymce-mobile-icon-large-font::before,
.tinymce-mobile-icon-style-formats::before {
  content: "\e264";
}

.tinymce-mobile-icon-undo::before {
  content: "\e166";
}

.tinymce-mobile-icon-redo::before {
  content: "\e15a";
}

.tinymce-mobile-icon-removeformat::before {
  content: "\e239";
}

.tinymce-mobile-icon-small-font::before {
  content: "\e906";
}

.tinymce-mobile-icon-readonly-back::before,
.tinymce-mobile-format-matches::after {
  content: "\e5ca";
}

.tinymce-mobile-icon-small-heading::before {
  content: "small";
}

.tinymce-mobile-icon-large-heading::before {
  content: "large";
}

.tinymce-mobile-icon-small-heading::before,
.tinymce-mobile-icon-large-heading::before {
  font-family: sans-serif;
  font-size: 80%;
}

.tinymce-mobile-mask-edit-icon::before {
  content: "\e254";
}

.tinymce-mobile-icon-back::before {
  content: "\e5c4";
}

.tinymce-mobile-icon-heading::before {
  /* TODO: Translate */
  content: "Headings";
  font-family: sans-serif;
  font-size: 80%;
  font-weight: bold;
}

.tinymce-mobile-icon-h1::before {
  content: "H1";
  font-weight: bold;
}

.tinymce-mobile-icon-h2::before {
  content: "H2";
  font-weight: bold;
}

.tinymce-mobile-icon-h3::before {
  content: "H3";
  font-weight: bold;
}
