import { Fun } from '@ephox/katamari';

export const Keys = {
  tab: Fun.constant(9),
  escape: Fun.constant(27),
  enter: Fun.constant(13),
  backspace: Fun.constant(8),
  delete: Fun.constant(46),
  left: Fun.constant(37),
  up: Fun.constant(38),
  right: Fun.constant(39),
  down: Fun.constant(40),
  space: Fun.constant(32),
  home: Fun.constant(36),
  end: Fun.constant(35),
  pageUp: Fun.constant(33),
  pageDown: Fun.constant(34)
};
