//
// Copyright (c) Tiny Technologies, Inc. All rights reserved.
// Licensed under the LGPL or a commercial license.
// For LGPL see License.txt in the project root for license information.
// For commercial licenses see https://www.tiny.cloud/
//

//
// Help Dialog
//

.tox {
  .tox-help__more-link {
    list-style: none;
    margin-top: 1em;
  }
}
