//
// Copyright (c) Tiny Technologies, Inc. All rights reserved.
// Licensed under the LGPL or a commercial license.
// For LGPL see License.txt in the project root for license information.
// For commercial licenses see https://www.tiny.cloud/
//

//
// Cursors
//

@cursor-format-painter-path: '../../../../svg/format-painter-cursor.svg';

.tox-cursor-format-painter {
  cursor: data-uri('image/svg+xml;charset=UTF-8', @cursor-format-painter-path), default;
}
