/**
 * Returns the current browser name.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @property browser.current
 * @type String
 */

/**
 * Returns the current browser major and minor version.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @property browser.version
 * @type Object
 */

/**
 * Returns <code>true</code> if the user's browser is Microsoft Edge.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @method browser.isEdge
 * @return {Boolean} Returns <code>true</code> if the user's browser is Microsoft Edge.
 */

/**
 * Returns <code>true</code> if the user's browser is Google Chrome.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @method browser.isChrome
 * @return {Boolean} Returns <code>true</code> if the user's browser is Google Chrome.
 */

/**
 * Returns <code>true</code> if the user's browser is Microsoft Internet Explorer.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @method browser.isIE
 * @return {Boolean} Returns <code>true</code> if the user's browser is Microsoft Internet Explorer.
 */

/**
 * Returns <code>true</code> if the user's browser is Opera.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @method browser.isOpera
 * @return {Boolean} Returns <code>true</code> if the user's browser is Opera.
 */

/**
 * Returns <code>true</code> if the user's browser is Firefox.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @method browser.isFirefox
 * @return {Boolean} Returns <code>true</code> if the user's browser is Firefox.
 */

/**
 * Returns <code>true</code> if the user's browser is Safari.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @method browser.isSafari
 * @return {Boolean} Returns <code>true</code> if the user's browser is Safari.
 */

/**
 * Returns the current operating system name.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @property os.current
 * @type String
 */

/**
 * Returns the current operating system major and minor version.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @property os.version
 * @type Object
 */

/**
 * Returns <code>true</code> if the user's operating system is Microsoft Windows.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @method os.isWindows
 * @return {Boolean} Returns <code>true</code> if the user's operating system is Microsoft Windows.
 */

/**
 * Returns <code>true</code> if the user's operating system is iOS.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @method os.isiOS
 * @return {Boolean} Returns <code>true</code> if the user's operating system is iOS.
 */

/**
 * Returns <code>true</code> if the user's operating system is Android.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @method os.isAndroid
 * @return {Boolean} Returns <code>true</code> if the user's operating system is Android.
 */

/**
 * Returns <code>true</code> if the user's operating system is Mac OS X.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @method os.isOSX
 * @return {Boolean} Returns <code>true</code> if the user's operating system is Mac OS X.
 */

/**
 * Returns <code>true</code> if the user's operating system is Linux.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @method os.isLinux
 * @return {Boolean} Returns <code>true</code> if the user's operating system is Linux.
 */

/**
 * Returns <code>true</code> if the user's operating system is Solaris.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @method os.isSolaris
 * @return {Boolean} Returns <code>true</code> if the user's operating system is Solaris.
 */

/**
 * Returns <code>true</code> if the user's operating system is FreeBSD.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @method os.isFreeBSD
 * @return {Boolean} Returns <code>true</code> if the user's operating system is FreeBSD.
 */

/**
 * Returns <code>true</code> if the user's operating system is ChromeOS.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @method os.isChromeOS
 * @return {Boolean} Returns <code>true</code> if the user's operating system is ChromeOS.
 */

/**
 * Returns <code>true</code> if the user's device is a desktop device.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @method deviceType.isDesktop
 * @return {Boolean} Returns <code>true</code> if the user's device is a desktop device.
 */

/**
 * Returns <code>true</code> if the user's device is an iPad.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @method deviceType.isiPad
 * @return {Boolean} Returns <code>true</code> if the user's device is an iPad.
 */

/**
 * Returns <code>true</code> if the user's device is an iPhone.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @method deviceType.isiPhone
 * @return {Boolean} Returns <code>true</code> if the user's device is an iPhone.
 */

/**
 * Returns <code>true</code> if the user's device is a phone.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @method deviceType.isPhone
 * @return {Boolean} Returns <code>true</code> if the user's device is a phone.
 */

/**
 * Returns <code>true</code> if the user's device is a tablet.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @method deviceType.isTablet
 * @return {Boolean} Returns <code>true</code> if the user's device is a tablet.
 */

/**
 * Returns <code>true</code> if the user's device is a touch device.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @method deviceType.isTouch
 * @return {Boolean} Returns <code>true</code> if the user's device is a touch device.
 */

/**
 * Returns <code>true</code> if the user's device is a WebView device.
 * <br>
 * <em>Added in TinyMCE 5.1</em>
 *
 * @method deviceType.isWebView
 * @return {Boolean} Returns <code>true</code> if the user's device is a WebView device.
 */
