//
// Copyright (c) Tiny Technologies, Inc. All rights reserved.
// Licensed under the LGPL or a commercial license.
// For LGPL see License.txt in the project root for license information.
// For commercial licenses see https://www.tiny.cloud/
//

//
// Visual characters
//

.mce-nbsp,
.mce-shy {
  background: #aaa;
}

.mce-shy::after {
  content: '-';
}
