/**
 * Copyright (c) Tiny Technologies, Inc. All rights reserved.
 * Licensed under the LGPL or a commercial license.
 * For LGPL see License.txt in the project root for license information.
 * For commercial licenses see https://www.tiny.cloud/
 */

export const enum ToolbarButtonClasses {
  Button = 'tox-tbtn',
  IconWrap = 'tox-tbtn__icon-wrap',
  Icon = 'tox-icon',
  Ticked = 'tox-tbtn--enabled',
  SplitButton = 'tox-split-button',
  MatchWidth = 'tox-tbtn--select'
}

export const enum MenuButtonClasses {
  Button = 'tox-mbtn',
  Active = 'tox-mbtn--active'
}
