//
// Copyright (c) Tiny Technologies, Inc. All rights reserved.
// Licensed under the LGPL or a commercial license.
// For LGPL see License.txt in the project root for license information.
// For commercial licenses see https://www.tiny.cloud/
//

/* RESET all the things! */
.tinymce-mobile-outer-container {
  all: initial;
  display: block;

  * {
    border: 0;
    box-sizing: initial;
    cursor: inherit;
    float: none;
    line-height: 1;
    margin: 0;
    outline: 0;
    padding: 0;
    -webkit-tap-highlight-color: transparent; /* TBIO-3691, stop the gray flicker on touch. */
    text-shadow: none;
    white-space: nowrap;
  }
}
