/**
 * Copyright (c) Tiny Technologies, Inc. All rights reserved.
 * Licensed under the LGPL or a commercial license.
 * For LGPL see License.txt in the project root for license information.
 * For commercial licenses see https://www.tiny.cloud/
 */

const saveState = 'save-state';
const disable = 'disable';
const enable = 'enable';

// TODO: dedupe these from ImageToolsEvents.ts in silver

export {
  saveState,
  disable,
  enable
};
