//
// Copyright (c) Tiny Technologies, Inc. All rights reserved.
// Licensed under the LGPL or a commercial license.
// For LGPL see License.txt in the project root for license information.
// For commercial licenses see https://www.tiny.cloud/
//

@tinymce-blue: #207ab7;
@color-black: #222f3e;
@toolstrip-background-color: #fff;
@toolstrip-border-color: darken(@toolstrip-background-color, 20);
@toolstrip-foreground-color:  #455a64;
@toolstrip-dark-background-color: @tinymce-blue;
@toolstrip-dark-color: #eceff1;
@toolstrip-input-background-color: lighten(@toolstrip-background-color, 20%);
@toolstrip-slider-background-color: darken(@toolstrip-background-color, 20%);
@toolstrip-input-foreground-color: @toolstrip-foreground-color;
@toolstrip-input-placeholder-foreground-color: #888;
@toolstrip-input-font-family: Sans-serif;
@toolstrip-input-icons-color: @toolstrip-input-placeholder-foreground-color;
@toolstrip-button-selected-background-color: multiply(white, contrast(@toolstrip-background-color, fade(@color-black, 25), fade(@color-black, 65)));
@toolstrip-button-selected-color: @toolstrip-slider-background-color;
@toolstrip-thumb-background-color: @toolstrip-foreground-color;
@toolstrip-button-horizontal-padding: .5em;

@dropup-background-color: white;

@tinymce-mobile-mask-menu-zindex: 1;

@mask-button-foreground-color: white;
@mask-button-background-color: @tinymce-blue;

@quick-animation-time: .15s;

@android-background-color: rgba(255, 255, 255, 1);

@editor-mask-background-color: #333;

@mask-button-font-size-small: 1.2em;
@mask-button-font-size-large: 1em;

@mask-button-size: 2.1em;

@tablets: ~"only screen and (min-device-width:700px)";
@iphone-small-landscape: ~"only screen and (min-device-width : 320px) and (max-device-width : 568px) and (orientation : landscape)";
@landscape: ~"only screen and (orientation: landscape)";
