//
// Copyright (c) Tiny Technologies, Inc. All rights reserved.
// Licensed under the LGPL or a commercial license.
// For LGPL see License.txt in the project root for license information.
// For commercial licenses see https://www.tiny.cloud/
//

/* stylelint-disable */

/* http://prismjs.com/ */

& when (@content-ui-darkmode = true) {
  @import (less) 'codesample-dark.css';
}

& when (@content-ui-darkmode = false) {
  @import (less) 'codesample-light.css';
}


/* stylelint-enable */
