//
// Copyright (c) Tiny Technologies, Inc. All rights reserved.
// Licensed under the LGPL or a commercial license.
// For LGPL see License.txt in the project root for license information.
// For commercial licenses see https://www.tiny.cloud/
//

//
// Global styling
//

@tinymce-border-color: @border-color;
@tinymce-border-radius: 0;
@tinymce-border-width: 1px;
@tinymce-box-shadow: none;
@editor-header-inline-background-color: if((alpha(@toolbar-background-color) > 0), transparent, @background-color); // If @toolbar-background-color is set to transparent, then set the inline toolbar background color to be another color to not be transparent over content.

.tox-tinymce {
  border: @tinymce-border-width solid @tinymce-border-color;
  border-radius: @tinymce-border-radius;
  box-shadow: @tinymce-box-shadow;
  box-sizing: border-box;
  display: flex;
  flex-direction: column;
  font-family: @font-stack;
  overflow: hidden;
  position: relative;
  visibility: inherit !important;
}

// Place the border/shadow on the header in inline mode instead of the root component
.tox-tinymce-inline {
  border: none;
  box-shadow: none;

  .tox-editor-header {
    background-color: @editor-header-inline-background-color;
    border: @tinymce-border-width solid @tinymce-border-color;
    border-radius: @tinymce-border-radius;
    box-shadow: @tinymce-box-shadow;
  }
}

.tox-tinymce-aux {
  font-family: @font-stack;
  z-index: @z-index-sink;
}

// Reset focus states
.tox-tinymce *:focus,
.tox-tinymce-aux *:focus {
  outline: none;
}

// Reset firefox focus states
button::-moz-focus-inner {
  border: 0;
}

// RTL
.tox[dir=rtl] {
  // Rotates configured icons 180 degrees when rendering in RTL
  .tox-icon--flip svg {
    transform: rotateY(180deg);
  }
}
