//
// Copyright (c) Tiny Technologies, Inc. All rights reserved.
// Licensed under the LGPL or a commercial license.
// For LGPL see License.txt in the project root for license information.
// For commercial licenses see https://www.tiny.cloud/
//

//
// Comments
//

@document-comment-background-color: #fff0b7;
@document-comment-active-background-color: #ffe168;

.tox-comments-visible .tox-comment {
  background-color: @document-comment-background-color;
}

.tox-comments-visible .tox-comment--active {
  background-color: @document-comment-active-background-color;
}
