export interface Hex {
  readonly value: string;
}

export interface Hsv {
  readonly hue: number;
  readonly saturation: number;
  readonly value: number;
}

export interface Rgba {
  readonly red: number;
  readonly green: number;
  readonly blue: number;
  readonly alpha: number;
}
