//
// Copyright (c) Tiny Technologies, Inc. All rights reserved.
// Licensed under the LGPL or a commercial license.
// For LGPL see License.txt in the project root for license information.
// For commercial licenses see https://www.tiny.cloud/
//

//
// Inline edit area
//

@inline-edit-area-border-color: @border-color;
@inline-edit-area-border-style: dotted;
@inline-edit-area-border-width: 1px;

.tox.tox-inline-edit-area {
  border: @inline-edit-area-border-width @inline-edit-area-border-style @inline-edit-area-border-color;
}
