//
// Copyright (c) Tiny Technologies, Inc. All rights reserved.
// Licensed under the LGPL or a commercial license.
// For LGPL see License.txt in the project root for license information.
// For commercial licenses see https://www.tiny.cloud/
//

//
// Search and replace
//

.mce-match-marker {
  background: #aaa;
  color: #fff;
}

.mce-match-marker-selected {
  &::selection {
    background: #39f;
    color: #fff;
  }

  background: #39f;
  color: #fff;
}
