/**
 * Copyright (c) Tiny Technologies, Inc. All rights reserved.
 * Licensed under the LGPL or a commercial license.
 * For LGPL see License.txt in the project root for license information.
 * For commercial licenses see https://www.tiny.cloud/
 */

const verticalAlignValues = [
  {
    text: 'None',
    value: ''
  },
  {
    text: 'Top',
    value: 'top'
  },
  {
    text: 'Middle',
    value: 'middle'
  },
  {
    text: 'Bottom',
    value: 'bottom'
  }
];

export {
  verticalAlignValues
};
