import { SidebarInstanceApi, SidebarSpec } from '../components/sidebar/Sidebar';

// These are the types that are exposed though a public end user api

export {
  SidebarSpec,
  SidebarInstanceApi
};
