/**
 * Copyright (c) Tiny Technologies, Inc. All rights reserved.
 * Licensed under the LGPL or a commercial license.
 * For LGPL see License.txt in the project root for license information.
 * For commercial licenses see https://www.tiny.cloud/
 */

//
// Place your HTML here
//

body {
  background-color: #2f3742;
  color: #dfe0e4;
  font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, Oxygen, Ubuntu, Cantarell, 'Open Sans', 'Helvetica Neue', sans-serif;
  line-height: 1.4;
  margin: 1rem;
}

a {
  color: #4099ff;
}

table {
  border-collapse: collapse;
}

/* Apply a default padding if legacy cellpadding attribute is missing */
table:not([cellpadding]) th,
table:not([cellpadding]) td {
  padding: .4rem;
}

/* Set default table styles if a table has a positive border attribute
   and no inline css */
table[border]:not([border="0"]):not([style*="border-width"]) th,
table[border]:not([border="0"]):not([style*="border-width"]) td {
  border-width: 1px;
}

/* Set default table styles if a table has a positive border attribute
   and no inline css */
table[border]:not([border="0"]):not([style*="border-style"]) th,
table[border]:not([border="0"]):not([style*="border-style"]) td {
  border-style: solid;
}

/* Set default table styles if a table has a positive border attribute
   and no inline css */
table[border]:not([border="0"]):not([style*="border-color"]) th,
table[border]:not([border="0"]):not([style*="border-color"]) td {
  border-color: #6d737b;
}

figure {
  display: table;
  margin: 1rem auto;
}

figure figcaption {
  color: #8a8f97;
  display: block;
  margin-top: .25rem;
  text-align: center;
}

hr {
  border-color: #6d737b;
  border-style: solid;
  border-width: 1px 0 0 0;
}

code {
  background-color: #6d737b;
  border-radius: 3px;
  padding: .1rem .2rem;
}

.mce-content-body:not([dir=rtl]) {
  blockquote {
    border-left: 2px solid #6d737b;
    margin-left: 1.5rem;
    padding-left: 1rem;
  }
}

// RTL
.mce-content-body[dir=rtl] {
  blockquote {
    border-right: 2px solid #6d737b;
    margin-right: 1.5rem;
    padding-right: 1rem;
  }
}
