const BACKSPACE: ReadonlyArray<number> = [ 8 ];
const TAB: ReadonlyArray<number> = [ 9 ];
const ENTER: ReadonlyArray<number> = [ 13 ];
const SHIFT: ReadonlyArray<number> = [ 16 ];
const CTRL: ReadonlyArray<number> = [ 17 ];
const ALT: ReadonlyArray<number> = [ 18 ];
const CAPSLOCK: ReadonlyArray<number> = [ 20 ];
const ESCAPE: ReadonlyArray<number> = [ 27 ];
const SPACE: ReadonlyArray<number> = [ 32 ];
const PAGEUP: ReadonlyArray<number> = [ 33 ];
const PAGEDOWN: ReadonlyArray<number> = [ 34 ];
const END: ReadonlyArray<number> = [ 35 ];
const HOME: ReadonlyArray<number> = [ 36 ];
const LEFT: ReadonlyArray<number> = [ 37 ];
const UP: ReadonlyArray<number> = [ 38 ];
const RIGHT: ReadonlyArray<number> = [ 39 ];
const DOWN: ReadonlyArray<number> = [ 40 ];
const INSERT: ReadonlyArray<number> = [ 45 ];
const DEL: ReadonlyArray<number> = [ 46 ];
const META: ReadonlyArray<number> = [ 91, 93, 224 ];
const F10: ReadonlyArray<number> = [ 121 ];

export {
  BACKSPACE,
  TAB,
  ENTER,
  SHIFT,
  CTRL,
  ALT,
  CAPSLOCK,
  ESCAPE,
  SPACE,
  PAGEUP,
  PAGEDOWN,
  END,
  HOME,
  LEFT,
  UP,
  RIGHT,
  DOWN,
  INSERT,
  DEL,
  META,
  F10
};
