import { Fun } from '@ephox/katamari';

const dismissPopups = Fun.constant('dismiss.popups');
const repositionPopups = Fun.constant('reposition.popups');
const mouseReleased = Fun.constant('mouse.released');

export {
  dismissPopups,
  repositionPopups,
  mouseReleased
};
