//
// Copyright (c) Tiny Technologies, Inc. All rights reserved.
// Licensed under the LGPL or a commercial license.
// For LGPL see License.txt in the project root for license information.
// For commercial licenses see https://www.tiny.cloud/
//

//
// Throbber
//

@throbber-busy-background-color: fade(@background-color, 60);

.tox {
  .tox-throbber {
    z-index: @z-index-throbber;
  }

  .tox-throbber__busy-spinner {
    align-items: center;
    background-color: @throbber-busy-background-color;
    bottom: 0;
    display: flex;
    justify-content: center;
    left: 0;
    position: absolute;
    right: 0;
    top: 0;
  }
}
